/*
 * Copyright (C) 2011-2017 Hottinger Bruel & Kjaer Benelux B.V.
 * Schutweg 15a
 * 5145NP Waalwijk
 * The Netherlands
 * http://www.hbm.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 *(at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * HBM IM3 FPGA button driver.
 *
 */
/* enable printing debugging info */
/* #define DEBUG */

#include <linux/kernel.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/interrupt.h>
#include <linux/cdev.h>
#include <linux/fs.h>
#include <linux/kfifo.h>
#include <linux/bug.h>
#include <linux/gfp.h>
#include <linux/sched.h>
#include <linux/version.h>
#include <linux/delay.h>

#include "hbm-fpga.h"
#include "hbm-fpga-driver.h"
#include "hbm-fpga-core.h"
#include "hbm-fpga-button.h"


struct button_t {
	/* FPGA instance */
	struct hbm_fpga_dev  *fpga;
};


void button_handle_isr(struct hbm_fpga_dev *fpga, unsigned int irq_status)
{
	uint32_t   DISPCTRL;
	irq_event  event;

	if (!(irq_status & BKP_INT_BUTTON_MASK))
		return;

	/* Clear the pending interrupt */
	DISPCTRL = fpga->regs->system.DISPCTRL;

	/* Fill out an event */
	event.type = BKP_INT_BUTTON;
	event.u.button.key = KEY_FRONTPANEL;
	event.u.button.state = (DISPCTRL & DISPCTRL__BUTTON_STATUS) ? 1 : 0;

	hbm_fpga_queue_event(fpga, &event);
}


int button_setup(struct hbm_fpga_dev *fpga)
{
	int              ret = 0;
	struct button_t  *button;

	/* Allocate memory for driver instance data */
	if (!(button = kzalloc(sizeof(struct button_t), GFP_KERNEL))) {
		ret = -ENOMEM;
		dev_err(&fpga->pci_dev->dev, "Error allocating button instance\n");
		goto err_mem;
	}

	/* Fill out our instance pointer for the caller */
	fpga->button = button;

	/* Store fpga instance instance for later reference */
	button->fpga = fpga;

	/* Enable button subsytem interrupt */
	hbm_fpga_change_irq_mask(fpga, BKP_INT_BUTTON_MASK, 0);

	return 0;

	fpga->button = NULL;
	kfree(button);
err_mem:
	return ret;
}


void button_remove(struct hbm_fpga_dev *fpga)
{
	struct button_t  *button = fpga->button;

	/* Disable button subsytem interrupt */
	hbm_fpga_change_irq_mask(fpga, 0, BKP_INT_BUTTON_MASK);

	fpga->button = NULL;
	kfree(button);
}

