#!/usr/bin/env bash
# Copyright (C) 2006-2012 OpenWrt.org
#------------------------------------------------------------------------------
#-- Copyright (C) 2020 Hottinger Bruel & Kjaer Benelux B.V.
#-- Schutweg 15a
#-- 5145 NP Waalwijk
#-- The Netherlands
#-- http://www.hbm.com
#-- All rights reserved
#--
#-- The copyright to the computer program(s) herein is the property of
#-- Hottinger Bruel & Kjaer Benelux B.V. , Netherlands. The code/program(s)
#-- may be used and/or copied only with the written permission of Hottinger Bruel & Kjaer Benelux B.V.
#-- or in accordance with the terms and conditions stipulated in the
#-- agreement/contract under which the program(s) have been supplied.
#-- This copyright notice must not be removed.
#--
#-- This Source Code is subject to the
#-- "General supply and license conditions for software"
#-- which is part of the standard terms and conditions of sale from HBM.
#------------------------------------------------------------------------------
set -x
[ $# == 10 -o $# == 11 ] || {
    echo "SYNTAX: $0 <file> <kernel size> <kernel directory> <rootfs size> <rootfs image> <align>"
    exit 1
}

#------------------------------------------------------------------------------
#-- Get Arguments
#------------------------------------------------------------------------------
OUTPUT="$1"
KERNELSIZE="$2"
KERNELDIR="$3"
ROOTFSSIZE="$4"
ROOTFSDIR="$5"
ALIGN="$6"

BOOTNUM="$7"
ROOTFSNUM="$8"
SETTINGSNUM="$9"
LOCALNUM="${10}"
CHECKSUMFILE="${11}"

rm -f "$OUTPUT"

head=16
sect=63
cyl=$(( ($KERNELSIZE + $ROOTFSSIZE + $ROOTFSSIZE) * 1024 * 1024 / ($head * $sect * 512)))

#------------------------------------------------------------------------------
#-- Create partition table
#------------------------------------------------------------------------------
set `ptgen -o "$OUTPUT" -h $head -s $sect -p ${KERNELSIZE}k -p ${ROOTFSSIZE}m -p ${ROOTFSSIZE}m ${ALIGN:+-l $ALIGN} ${SIGNATURE:+-S 0x$SIGNATURE}`

KERNELOFFSET="$(($1 / 512))"
KERNELSIZE="$2"
ROOTFSOFFSET0="$(($3 / 512))"
ROOTFSSIZE0="$4"
ROOTFSOFFSET1="$(($5 / 512))"
ROOTFSSIZE1="$6"

[ -n "$PADDING" ] && dd if=/dev/zero of="$OUTPUT" bs=512 seek="$ROOTFSOFFSET0" conv=notrunc count="$ROOTFSSIZE0"

#------------------------------------------------------------------------------
#-- Create the disk layout UCI config
#------------------------------------------------------------------------------
install -d ${KERNELDIR}/config
cat > ${KERNELDIR}/config/disklayout <<__SOMEREALLYUNIQUESTRING__
config bootpart boot
	option 'number' 	$BOOTNUM
	option 'fstype'	'ext2'

config slotpart slot0
	option 'number'		$ROOTFSNUM
	option 'slot_number'	'0'
	option 'fstype'	'ext4'

config slotpart slot1
	option 'number'		$(( $ROOTFSNUM + 1 ))
	option 'slot_number'	'1'
	option 'fstype'	'ext4'

config settingspart settings
	option 'number'		$SETTINGSNUM
	option 'fstype'	'ext4'
	option 'legacy_fstype' 'ext2'

config lspart ls
	option 'number'		$LOCALNUM
	option 'fstype'	'ext4'

__SOMEREALLYUNIQUESTRING__

#------------------------------------------------------------------------------
#-- Create file to collect all system partition checksums
#------------------------------------------------------------------------------
rm -f $CHECKSUMFILE
touch $CHECKSUMFILE

#------------------------------------------------------------------------------
#-- Create Boot (Grub) Partition
#------------------------------------------------------------------------------
make_ext4fs -J -l "$KERNELSIZE" "$OUTPUT.kernel" "$KERNELDIR"
dd if="$OUTPUT.kernel" of="$OUTPUT" bs=512 seek="$KERNELOFFSET" conv=notrunc
cat "$OUTPUT.kernel" | md5sum | sed 's/-/boot/' > "$CHECKSUMFILE"
rm -f "$OUTPUT.kernel"

#------------------------------------------------------------------------------
#-- Create RootFs Slot0 Partition
#------------------------------------------------------------------------------
make_ext4fs -J -l "$ROOTFSSIZE0" -L "slot0" "$OUTPUT.linux" "$ROOTFSDIR"
dd if="$OUTPUT.linux" of="$OUTPUT" bs=512 seek="$ROOTFSOFFSET0" conv=notrunc
cat "$OUTPUT.linux" | md5sum | sed 's/-/slot0/' >> "$CHECKSUMFILE"
rm -f "$OUTPUT.linux"

#------------------------------------------------------------------------------
#-- Create RootFs Slot1 Partition
#------------------------------------------------------------------------------
make_ext4fs -J -l "$ROOTFSSIZE1" -L "slot1" "$OUTPUT.linux" "$ROOTFSDIR"
dd if="$OUTPUT.linux" of="$OUTPUT" bs=512 seek="$ROOTFSOFFSET1" conv=notrunc
cat "$OUTPUT.linux" | md5sum | sed 's/-/slot1/' >> "$CHECKSUMFILE"
rm -f "$OUTPUT.linux"


