#!/usr/bin/env bash
# Copyright (C) 2006-2012 OpenWrt.org
#------------------------------------------------------------------------------
#-- Copyright (C) 2020 Hottinger Bruel & Kjaer Benelux B.V.
#-- Schutweg 15a
#-- 5145 NP Waalwijk
#-- The Netherlands
#-- http://www.hbm.com
#-- All rights reserved
#--
#-- The copyright to the computer program(s) herein is the property of
#-- Hottinger Bruel & Kjaer Benelux B.V. , Netherlands. The code/program(s)
#-- may be used and/or copied only with the written permission of Hottinger Bruel & Kjaer Benelux B.V.
#-- or in accordance with the terms and conditions stipulated in the
#-- agreement/contract under which the program(s) have been supplied.
#-- This copyright notice must not be removed.
#--
#-- This Source Code is subject to the
#-- "General supply and license conditions for software"
#-- which is part of the standard terms and conditions of sale from HBM.
#------------------------------------------------------------------------------

set -x
[ $# == 6 -o $# == 8 ] || {
    echo "SYNTAX: $0 <file> <kernel size> <kernel directory> <rootfs size> <rootfs > <partition size> <partions directory> [<align>]"
    exit 1
}

#------------------------------------------------------------------------------
#-- Get Arguments
#------------------------------------------------------------------------------
OUTPUT="$1"
KERNELSIZE="$2"
KERNELDIR="$3"
ROOTFSSIZE="$4"
ROOTFSDIR="$5"
PARTFSSIZE="$6"
PARTFSDIR="$7"
ALIGN="$8"

rm -f "$OUTPUT"

head=16
sect=63
cyl=$(( ($KERNELSIZE + $ROOTFSSIZE + $PARTFSSIZE) * 1024 * 1024 / ($head * $sect * 512)))

#------------------------------------------------------------------------------
#-- Create Partition Table
#------------------------------------------------------------------------------
set `ptgen -o "$OUTPUT" -h $head -s $sect -p ${KERNELSIZE}m -p ${ROOTFSSIZE}m -p ${PARTFSSIZE}m ${ALIGN:+-l $ALIGN} ${SIGNATURE:+-S 0x$SIGNATURE}`

KERNELOFFSET="$(($1 / 512))"
KERNELSIZE="$2"
ROOTFSOFFSET="$(($3 / 512))"
ROOTFSSIZE="$4"
PARTFSOFFSET="$(($5 / 512))"
PARTFSSIZE="$6"

[ -n "$PADDING" ] && dd if=/dev/zero of="$OUTPUT" bs=512 seek="$ROOTFSOFFSET" conv=notrunc count="$ROOTFSSIZE"

#------------------------------------------------------------------------------
#-- Create the HBM specific slot config file
#------------------------------------------------------------------------------
SETTINGS_FILE=settings_partition.tgz
SETTINGS_DIR=`mktemp -d`

install -d ${SETTINGS_DIR}/boot -m 0700

cat > ${SETTINGS_DIR}/boot/slot <<__SOMEREALLYUNIQUESTRING2__
config 'slot_cfg' 'slot0'
	option 'valid' '1'

config 'slot_cfg' 'slot1'
	option 'valid' '1'

config '0' 'active'

__SOMEREALLYUNIQUESTRING2__

# Create md5 file for this config file
md5sum ${SETTINGS_DIR}/boot/slot | awk '{printf "%s", $1}' > ${SETTINGS_DIR}/boot/slot.md5
echo "  /mnt/settings/boot/slot" >> ${SETTINGS_DIR}/boot/slot.md5

# Create the /mnt/settings/etc settings directory
install -d ${SETTINGS_DIR}/etc -m 0700

# Create the /mnt/settings/etc/samba (private) settings directory
install -d ${SETTINGS_DIR}/etc/samba -m 0700

# Create an empty smbpasswd file
touch ${SETTINGS_DIR}/etc/samba/smbpasswd
chmod 0600 ${SETTINGS_DIR}/etc/samba/smbpasswd
chown 0:0 ${SETTINGS_DIR}/etc/samba/smbpasswd

# Create an empty username.map file
touch ${SETTINGS_DIR}/etc/samba/username.map
chmod 0600 ${SETTINGS_DIR}/etc/samba/username.map
chown 0:0 ${SETTINGS_DIR}/etc/samba/username.map

tar cvzf $SETTINGS_FILE -C $SETTINGS_DIR --numeric-owner --owner=0 --group=0 .
md5sum ${SETTINGS_FILE} | awk '{printf "%s", $1}' > ${SETTINGS_FILE}.md5
echo "  /tmp/hbmusb1/${SETTINGS_FILE}" >> ${SETTINGS_FILE}.md5
cp $SETTINGS_FILE $PARTFSDIR
cp $SETTINGS_FILE.md5 $PARTFSDIR

#------------------------------------------------------------------------------
#-- Copy Bios Images To Playload Partition
#------------------------------------------------------------------------------
BIOS_FILE=bios.tgz
cp $BIOS_FILE $PARTFSDIR

#------------------------------------------------------------------------------
#-- Create Boot (Grub) Partition
#------------------------------------------------------------------------------
make_ext4fs -J -l "$KERNELSIZE" -L "boot" "$OUTPUT.grub" "$KERNELDIR"
dd if="$OUTPUT.grub" of="$OUTPUT" bs=512 seek="$KERNELOFFSET" conv=notrunc
rm -f "$OUTPUT.grub"

#------------------------------------------------------------------------------
#-- Create USB RootFs Partition
#------------------------------------------------------------------------------
make_ext4fs -J -l "$ROOTFSSIZE" -L "hbmusb0" "$OUTPUT.linux" "$ROOTFSDIR"
dd if="$OUTPUT.linux" of="$OUTPUT" bs=512 seek="$ROOTFSOFFSET" conv=notrunc
rm -f "$OUTPUT.linux"

#------------------------------------------------------------------------------
#-- Create Payload Partition
#------------------------------------------------------------------------------
make_ext4fs -J -l "$PARTFSSIZE" -L "hbmusb1" "$OUTPUT.part" "$PARTFSDIR"
dd if="$OUTPUT.part" of="$OUTPUT" bs=512 seek="$PARTFSOFFSET" conv=notrunc
rm -f "$OUTPUT.part"


